package sun.awt.windows;

import java.awt.event.ItemEvent;
import java.awt.Window;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import sun.awt.SunToolkit;
import java.awt.Component;
import java.awt.FontMetrics;
import java.awt.Choice;
import java.awt.Dimension;
import java.awt.event.WindowListener;
import java.awt.peer.ChoicePeer;

final class WChoicePeer extends WComponentPeer implements ChoicePeer
{
    private WindowListener windowListener;
    
    @Override
    public Dimension getMinimumSize() {
        FontMetrics fontMetrics = this.getFontMetrics(((Choice)this.target).getFont());
        Choice choice = (Choice)this.target;
        int max = 0;
        int itemCount = choice.getItemCount();
        while (itemCount-- > 0) {
            max = Math.max(fontMetrics.stringWidth(choice.getItem(itemCount)), max);
        }
        return new Dimension(28 + max, Math.max(fontMetrics.getHeight() + 6, 15));
    }
    
    @Override
    public boolean isFocusable() {
        return true;
    }
    
    @Override
    public native void select(int p0);
    
    @Override
    public void add(String s, int n) {
        this.addItem(s, n);
    }
    
    @Override
    public boolean shouldClearRectBeforePaint() {
        return false;
    }
    
    @Override
    public native void removeAll();
    
    @Override
    public native void remove(int p0);
    
    public void addItem(String s, int n) {
        this.addItems(new String[] { s }, n);
    }
    
    public native void addItems(String[] p0, int p1);
    
    @Override
    public synchronized native void reshape(int p0, int p1, int p2, int p3);
    
    WChoicePeer(Choice choice) {
        super(choice);
    }
    
    @Override
    native void create(WComponentPeer p0);
    
    @Override
    void initialize() {
        Choice choice = (Choice)this.target;
        int itemCount = choice.getItemCount();
        if (itemCount > 0) {
            String[] array = new String[itemCount];
            for (int i = 0; i < itemCount; ++i) {
                array[i] = choice.getItem(i);
            }
            this.addItems(array, 0);
            if (choice.getSelectedIndex() >= 0) {
                this.select(choice.getSelectedIndex());
            }
        }
        Window containingWindow = SunToolkit.getContainingWindow((Component)this.target);
        if (containingWindow != null) {
            WWindowPeer wWindowPeer = (WWindowPeer)containingWindow.getPeer();
            if (wWindowPeer != null) {
                wWindowPeer.addWindowListener(this.windowListener = new WindowAdapter() {
                    @Override
                    public void windowIconified(WindowEvent windowEvent) {
                        WChoicePeer.this.closeList();
                    }
                    
                    @Override
                    public void windowClosing(WindowEvent windowEvent) {
                        WChoicePeer.this.closeList();
                    }
                });
            }
        }
        super.initialize();
    }
    
    @Override
    protected void disposeImpl() {
        Window containingWindow = SunToolkit.getContainingWindow((Component)this.target);
        if (containingWindow != null) {
            WWindowPeer wWindowPeer = (WWindowPeer)containingWindow.getPeer();
            if (wWindowPeer != null) {
                wWindowPeer.removeWindowListener(this.windowListener);
            }
        }
        super.disposeImpl();
    }
    
    void handleAction(final int n) {
        final Choice target = (Choice)this.target;
        SunToolkit.executeOnEventHandlerThread(target, new Runnable() {
            @Override
            public void run() {
                target.select(n);
                WChoicePeer.this.postEvent(new ItemEvent(target, 701, target.getItem(n), 1));
            }
        });
    }
    
    int getDropDownHeight() {
        Choice choice = (Choice)this.target;
        return this.getFontMetrics(choice.getFont()).getHeight() * Math.min(choice.getItemCount(), 8);
    }
    
    native void closeList();
}
